/*
	
	TITLE:		Gfx3D.cpp
	DESC:		Interface implementation for simple Direct3D9 sample program
	DATE:		January 2003

	AUTHOR:		Jack Hoxley

*/

//include system includes
#include <math.h>
#include <Windows.h>
#include <exception>
#include <d3dx9.h>
#include <stdio.h>

//include the implementation header
#include "Gfx3D.h"
#include "Gfx3DException.h"

//conditional statements:
#define DISPLAY_VSYNC		//comment out this #define to disable vsync'ing.

//used to create a windowed-mode Direct3D9 graphics engine
Gfx3D::Gfx3D(HWND hWnd) {
	//configure default values
	bWireFrame = false;
	bUseTextures = true;
	bCulling = true;
	bDrawFPS = true;

	bInitOkay = false;
	fCurrFPS = 0.0;
	iCurrCnt = 0;

	pD3D9 = NULL;
	pDev = NULL;
	pTexCube1 = NULL;
	pTexCube2 = NULL;
	pVBCube = NULL;
	pVBMenu = NULL;

	Cube1Angle = 0.0f;
	Cube2Angle = 0.0f;

	InitialiseDevice(hWnd,0,0,0,true);
}

//used to create a fullscreen Direct3D9 graphics engine
Gfx3D::Gfx3D(HWND hWnd, int iW, int iH, int iD) {
	//configure default values
	bWireFrame = false;
	bUseTextures = true;
	bCulling = true;
	bDrawFPS = true;

	bInitOkay = false;
	fCurrFPS = 0;
	iCurrCnt = 0;
	
	pD3D9 = NULL;
	pDev = NULL;
	pTexCube1 = NULL;
	pTexCube2 = NULL;
	pVBCube = NULL;

	Cube1Angle = 0.0f;
	Cube2Angle = 0.0f;
	
	if (IsDisplayModeOkay(iW,iH,iD)) {
		//hide the mouse cursor while in fullscreen mode
		do {} while (ShowCursor(false)>0);

		//set the ball rolling...
		InitialiseDevice(hWnd,iW,iH,iD,false);

	} else
		throw Gfx3DException("Resolution is not supported!","Gfx3D.cpp",__LINE__);
}

//general destructor, for cleaning up during termination
Gfx3D::~Gfx3D() {

	//free other memory
	if (sDispInfo != NULL)
		delete sDispInfo;

	if (sDevInfo != NULL)
		delete sDevInfo;

	//release the textures
	if (pTexMenu != NULL)
		pTexMenu->Release();

	if (pTexCube1 != NULL)
		pTexCube1->Release();

	if (pTexCube2 != NULL)
		pTexCube2->Release();

	//release the geometry
	if (pVBMenu != NULL)
		pVBMenu->Release();

	if (pVBCube != NULL)
		pVBCube->Release();

	//free the device
	if (pDev != NULL)
		pDev->Release();

	//free the D3D9 interface
	if (pD3D9 != NULL)
		pD3D9->Release();

	//show the mouse again
	do {} while (ShowCursor(true)<=0);
}

bool Gfx3D::IsDisplayModeOkay(int iW, int iH, int iD) {
	LPDIRECT3D9 pD3D;
	D3DFORMAT fmtTmp;

	pD3D = Direct3DCreate9( D3D_SDK_VERSION );
	if ( NULL == pD3D )
		return false;

	switch (iD) {
		case 16:
			fmtTmp = D3DFMT_R5G6B5;
			break;
		case 24:
			fmtTmp = D3DFMT_R8G8B8;
			break;
		case 32:
			fmtTmp = D3DFMT_X8R8G8B8;
			break;
		default:
			throw Gfx3DException("Invalid display depth specified","Gfx3D.cpp",__LINE__);
	}
	
	UINT cnt = pD3D->GetAdapterModeCount( D3DADAPTER_DEFAULT, fmtTmp);

	for (UINT i = 0; i < cnt; i++ ) {
		D3DDISPLAYMODE Mode;

		pD3D->EnumAdapterModes( D3DADAPTER_DEFAULT, fmtTmp, i, &Mode);

		if ( (Mode.Width == iW) && (Mode.Height == iH) ) {
			pD3D->Release();
			return true;
		}
	}
	
	pD3D->Release();
	return false;
}

bool Gfx3D::OneFrameRender() {
	//dont do any work if the device was not configured.
	if (!bInitOkay)
		return false;
	
	//clear any existing colour and depth information.
	pDev->Clear(0,NULL,D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER,D3DCOLOR_XRGB(0,0,64),1.0f,0);

	//tell D3D we're gonna start rendering
	pDev->BeginScene();

	//////////////////////////////
	//	render the two cubes	//
	//////////////////////////////
	pDev->SetStreamSource( 0, pVBCube, 0, sizeof(vertex3D) );
	pDev->SetFVF( FVF_VERTEX3D );

	//cube 1:
	if (bUseTextures)
		pDev->SetTexture( 0, pTexCube1 );
	else
		pDev->SetTexture( 0, NULL );
	pDev->SetTransform( D3DTS_WORLD, &MatCube1 );
	pDev->DrawPrimitive( D3DPT_TRIANGLELIST, 0, 12 );

	//cube 2:
	if (bUseTextures)
		pDev->SetTexture( 0, pTexCube2 );
	else
		pDev->SetTexture( 0, NULL );
	pDev->SetTransform( D3DTS_WORLD, &MatCube2 );
	pDev->DrawPrimitive( D3DPT_TRIANGLELIST, 0, 12 );

	//////////////////////////
	//	render the menu bar	//
	//////////////////////////
	if (bWireFrame)
		pDev->SetRenderState( D3DRS_FILLMODE, D3DFILL_SOLID );

	pDev->SetRenderState( D3DRS_ALPHABLENDENABLE, true );

	pDev->SetStreamSource( 0, pVBMenu, 0, sizeof(vertex2D) );
	pDev->SetFVF( FVF_VERTEX2D );
	pDev->SetTexture( 0, pTexMenu );
	pDev->DrawPrimitive( D3DPT_TRIANGLESTRIP, 0, 2 );

	if (bWireFrame)
		pDev->SetRenderState( D3DRS_FILLMODE, D3DFILL_WIREFRAME );

	pDev->SetRenderState( D3DRS_ALPHABLENDENABLE, false );

	//////////////////////////////
	//	render supporting text	//
	//////////////////////////////
	//render the menu bar text:
	RECT rcText;
	lpFont->Begin();
		rcText.left = 5;

		rcText.top = 3 + (5 * (iFontSize + 5 ));
		if (bWireFrame)
			lpFont->DrawTextA("F1 - Solid", -1, &rcText, DT_TOP|DT_LEFT, D3DCOLOR_XRGB(200,128,64) );	
		else
			lpFont->DrawTextA("F1 - Wireframe", -1, &rcText, DT_TOP|DT_LEFT, D3DCOLOR_XRGB(200,128,64) );	

		rcText.top = 3 + (6 * (iFontSize + 5 ));
		if (bUseTextures)
			lpFont->DrawTextA("F2 - Shaded", -1, &rcText, DT_TOP|DT_LEFT, D3DCOLOR_XRGB(200,128,64) );
		else
			lpFont->DrawTextA("F2 - Textured", -1, &rcText, DT_TOP|DT_LEFT, D3DCOLOR_XRGB(200,128,64) );

		rcText.top = 3 + (7 * (iFontSize + 5 ));
		if (bCulling)
			lpFont->DrawTextA("F3 - No Culling", -1, &rcText, DT_TOP|DT_LEFT, D3DCOLOR_XRGB(200,128,64) );
		else
			lpFont->DrawTextA("F3 - CCW Culling", -1, &rcText, DT_TOP|DT_LEFT, D3DCOLOR_XRGB(200,128,64) );

		rcText.top = 3 + (8 * (iFontSize + 5 ));
		if (bDrawFPS)
			lpFont->DrawTextA("F4 - Hide Status", -1, &rcText, DT_TOP|DT_LEFT, D3DCOLOR_XRGB(200,128,64) );
		else
			lpFont->DrawTextA("F4 - Show Status", -1, &rcText, DT_TOP|DT_LEFT, D3DCOLOR_XRGB(200,128,64) );
	lpFont->End();

	//only draw text if user selected it.
	if ( bDrawFPS ) {
		//required variables/allocations
		RECT rcTextFPS, rcTextRes, rcTextDev;
		char *pMsg = new char[100];

		//set the draw area for the frame-rate
		rcTextFPS.top = 5;		rcTextFPS.left = 5;

		//set the draw area for the resolution information
		rcTextRes.left = 5;		rcTextRes.top = 22;

		//set the draw area for the device information
		rcTextDev.left = 5;		rcTextDev.top = 39;		

		//all other strings are calculated in InitialiseDevice()
		sprintf( pMsg, "Current Frame Rate: %d", (int)fCurrFPS );

		//begin drawing the text.
		lpFont->Begin();
			lpFont->DrawTextA(pMsg,-1, &rcTextFPS, DT_TOP|DT_LEFT, D3DCOLOR_XRGB(255,200,0));
			lpFont->DrawTextA(sDispInfo,-1, &rcTextRes, DT_TOP|DT_LEFT, D3DCOLOR_XRGB(255,200,0));
			lpFont->DrawTextA(sDevInfo,-1, &rcTextDev, DT_TOP|DT_LEFT, D3DCOLOR_XRGB(255,200,0));
		lpFont->End();

		//clear up.
		delete pMsg;
	}
	
	//tell D3D we've finished sending geometry to be rendered.
	pDev->EndScene();
	
	//tell D3D to present the image we just built on the screen.
	//its at this point that the user will actually see something.
	pDev->Present(NULL,NULL,NULL,NULL);
	return true;
}

bool Gfx3D::OneFrameUpdate() {

	//variables
	D3DXMATRIXA16 matTemp;

	//calculate time difference
	Cube1Angle += ( (float)(GetTickCount() - dwLastFrameUpdate) / 1000 ) * Cube1Speed;
	Cube2Angle += ( (float)(GetTickCount() - dwLastFrameUpdate) / 1000 ) * Cube2Speed;

	dwLastFrameUpdate = GetTickCount();

	//calculate cube 1's matrix
	D3DXMatrixIdentity( &MatCube1 );
	
	D3DXMatrixIdentity( &matTemp );	
	D3DXMatrixScaling( &matTemp, 3, 3, 3 );
	D3DXMatrixMultiply( &MatCube1, &MatCube1, &matTemp );

	D3DXMatrixIdentity( &matTemp );
	D3DXMatrixRotationX( &matTemp, Cube1Angle*(D3DX_PI/180) );
	D3DXMatrixMultiply( &MatCube1, &MatCube1, &matTemp );

	D3DXMatrixIdentity( &matTemp );
	D3DXMatrixRotationY( &matTemp, Cube1Angle*(D3DX_PI/180) );
	D3DXMatrixMultiply( &MatCube1, &MatCube1, &matTemp );

	//calculate cube 2's matrix
	D3DXMatrixIdentity( &MatCube2 );
	
	D3DXMatrixIdentity( &matTemp );
	D3DXMatrixScaling( &matTemp, 4, 5, 0.75 );
	D3DXMatrixMultiply( &MatCube2, &MatCube2, &matTemp );

	D3DXMatrixIdentity( &matTemp );
	D3DXMatrixTranslation( &matTemp, -8, -4, 0 );
	D3DXMatrixMultiply( &MatCube2, &MatCube2, &matTemp );

	D3DXMatrixIdentity( &matTemp );
	D3DXMatrixRotationZ( &matTemp, Cube2Angle*(D3DX_PI/180) );
	D3DXMatrixMultiply( &MatCube2, &MatCube2, &matTemp );

	//calculate frame rate
	if ( ( GetTickCount() - dwLastFPSCheck ) >= iFPSPollRate ) {
		//time to calculate the frame rate
		dwLastFPSCheck = GetTickCount();
		fCurrFPS = (float)iCurrCnt * (1000/(float)iFPSPollRate);
		iCurrCnt = 0;
	}
	iCurrCnt++;

	return true;
}

//properties:

void Gfx3D::Wireframe(bool bWire) {
	if (!bInitOkay)
		return;

	bWireFrame = bWire;
	if (bWire)
		pDev->SetRenderState( D3DRS_FILLMODE, D3DFILL_WIREFRAME );
	else
		pDev->SetRenderState( D3DRS_FILLMODE, D3DFILL_SOLID );
}

bool Gfx3D::Wireframe() {
	return bWireFrame;
}

void Gfx3D::UseTextures(bool bTex) {
	bUseTextures = bTex;
}

bool Gfx3D::UseTextures() {
	return bUseTextures;
}

void Gfx3D::BackFaceCulling(bool bCull) {
	if (!bInitOkay)
		return;

	bCulling = bCull;
	if (bCull)
		pDev->SetRenderState( D3DRS_CULLMODE, D3DCULL_CCW );
	else
		pDev->SetRenderState( D3DRS_CULLMODE, D3DCULL_NONE );
}

bool Gfx3D::BackFaceCulling() {
	return bCulling;
}

void Gfx3D::DrawFrameRate(bool bFPS) {
	bDrawFPS = bFPS;
}

bool Gfx3D::DrawFrameRate() {
	return bDrawFPS;
}

//private functions
bool Gfx3D::InitialiseDevice(HWND hWnd, int iW, int iH, int iD, bool bWin) {
	
	pD3D9 = Direct3DCreate9( D3D_SDK_VERSION );
	if (pD3D9 == NULL)
		throw Gfx3DException("Could not create Direct3D9 interface","Gfx3D.cpp",__LINE__);

	D3DPRESENT_PARAMETERS pp;
	ZeroMemory( &pp, sizeof(pp) );
	pp.Windowed = bWin;
	pp.AutoDepthStencilFormat = D3DFMT_D16;
	pp.EnableAutoDepthStencil = true;
	pp.SwapEffect = D3DSWAPEFFECT_DISCARD;
	if (bWin) {
		pp.BackBufferFormat = D3DFMT_UNKNOWN;
	} else {
		pp.BackBufferWidth = iW;
		pp.BackBufferHeight = iH;
		switch (iD) {
			case 16:
				pp.BackBufferFormat = D3DFMT_R5G6B5;
				break;
			case 24:
				pp.BackBufferFormat = D3DFMT_R8G8B8;
				break;
			case 32:
				pp.BackBufferFormat = D3DFMT_X8R8G8B8;
				break;
			default:
				throw Gfx3DException("Incorrect display format specified","Gfx3D.cpp",__LINE__);
		}
 	}
#ifdef DISPLAY_VSYNC
	pp.PresentationInterval = D3DPRESENT_INTERVAL_ONE;			//vsync is on
#else
	pp.PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;	//vsync is off
#endif
	
	D3DCAPS9 caps;
	DWORD behaviour;
	pD3D9->GetDeviceCaps( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, &caps );
	//we want to select hardware transform and lighting if its available:
	if ( caps.DevCaps & D3DDEVCAPS_HWTRANSFORMANDLIGHT )
		behaviour = D3DCREATE_HARDWARE_VERTEXPROCESSING;
	else
		behaviour = D3DCREATE_SOFTWARE_VERTEXPROCESSING;

	if( FAILED( pD3D9->CreateDevice( D3DADAPTER_DEFAULT,
									 D3DDEVTYPE_HAL,
									 hWnd,
									 behaviour,
									 &pp,
									 &pDev ) ) )
		throw Gfx3DException("Could not create Direct3D9 device","Gfx3D.cpp",__LINE__);

	//retrieve information about the display mode:
	sDispInfo = new char[100];
	char *sFmt = new char[30];
	switch (pp.BackBufferFormat) {
		case D3DFMT_R5G6B5:
			sFmt = "16bit 565";
			break;
		case D3DFMT_X1R5G5B5:
			sFmt = "16bit X555";
			break;
		case D3DFMT_R8G8B8:
			sFmt = "24bit 888";
			break;
		case D3DFMT_X8R8G8B8:
			sFmt = "32bit X888";
			break;
		case D3DFMT_A8R8G8B8:
			sFmt = "32bit 8888";
			break;
		default:
			sFmt = "Unknown Format";
			break;
	}
	sprintf(	sDispInfo, "%dx%d (%s)",pp.BackBufferWidth, pp.BackBufferHeight, sFmt	);

	D3DADAPTER_IDENTIFIER9 *pID = new D3DADAPTER_IDENTIFIER9;
	pD3D9->GetAdapterIdentifier( D3DADAPTER_DEFAULT, 0, pID);
	sDevInfo = new char[200];
	sprintf(	sDevInfo, "%s (v%d.%d build %d)",pID->Description, 
										   LOWORD(pID->DriverVersion.HighPart),
										   HIWORD(pID->DriverVersion.LowPart),
										   LOWORD(pID->DriverVersion.LowPart)
										   );

	//setup basic render states
	pDev->SetRenderState( D3DRS_CULLMODE, D3DCULL_CCW );
	pDev->SetRenderState( D3DRS_LIGHTING, false );
	pDev->SetRenderState( D3DRS_ZENABLE, true );
	
	//set up texture information
	pDev->SetRenderState( D3DRS_SRCBLEND, D3DBLEND_SRCALPHA );
	pDev->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );
	pDev->SetSamplerState( 0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR );
	pDev->SetSamplerState( 0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR );

	pDev->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_MODULATE );
    pDev->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
    pDev->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );
    pDev->SetTextureStageState( 0, D3DTSS_ALPHAOP,   D3DTOP_MODULATE );
	
	//load relevent textures into memory
	if (!LoadTextures(pp.BackBufferFormat))
		throw Gfx3DException("Could not execute Gfx3D::LoadTextures()","Gfx3D.cpp",__LINE__);
	
	//load required geometry into memory
	if (!LoadGeometry())
		throw Gfx3DException("Could not execute Gfx3D::LoadGeometry()","Gfx3D.cpp",__LINE__);

	//configure the matrices
	D3DXMatrixIdentity( &MatWorld );
	D3DXMatrixLookAtLH( &MatView, new D3DXVECTOR3(0,0,-15), new D3DXVECTOR3(0,0,0), new D3DXVECTOR3(0,1,0) );
	D3DXMatrixPerspectiveFovLH( &MatProj, D3DX_PI/4, 4/3, 1, 100 );

	pDev->SetTransform ( D3DTS_WORLD, &MatWorld );
	pDev->SetTransform ( D3DTS_VIEW, &MatView );
	pDev->SetTransform ( D3DTS_PROJECTION, &MatProj );

	//configure the fonts
	hMainFont = CreateFont(iFontSize,0,0,0,FW_NORMAL,false,false,0,
						   ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,
						   DEFAULT_QUALITY, DEFAULT_PITCH, "Verdana");
	D3DXCreateFont( pDev, hMainFont, &lpFont );

	//get a base value for the first frames calculations
	dwLastFrameUpdate = GetTickCount();
	
	bInitOkay = true; //we succeeded this far!
	return true;
}

bool Gfx3D::LoadTextures(D3DFORMAT fmtDisplay) {

	// Attempt to load the menubar.bmp texture.
	if ( SUCCEEDED(pD3D9->CheckDeviceFormat( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, fmtDisplay, 0, D3DRTYPE_TEXTURE, D3DFMT_A8R8G8B8)) ) {
		//load menubar.bmp [256x128] magenta colorkey
		if ( FAILED( D3DXCreateTextureFromFileEx( pDev, "menubar.bmp",
												256, 128, 1,
												0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED,
												D3DX_FILTER_LINEAR, D3DX_FILTER_LINEAR,
												0xFFFF00FF, NULL, NULL, &pTexMenu ) ) )
			//error!
			throw Gfx3DException("Could not load menubar.bmp as a 32bit A888 Texture","Gfx3D.cpp",__LINE__);

	} else if ( SUCCEEDED( pD3D9->CheckDeviceFormat( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, fmtDisplay, 0, D3DRTYPE_TEXTURE, D3DFMT_A1R5G5B5) ) ) {
		//load menubar.bmp
		if ( FAILED( D3DXCreateTextureFromFileEx( pDev, "menubar.bmp",
												256, 128, 1,
												0, D3DFMT_A1R5G5B5, D3DPOOL_MANAGED,
												D3DX_FILTER_LINEAR, D3DX_FILTER_LINEAR,
												0xFFFF00FF, NULL, NULL, &pTexMenu ) ) )
			//error!
			throw Gfx3DException("Could not load menubar.bmp as a 16bit A555 Texture","Gfx3D.cpp",__LINE__);

	} else {
		//error!
		throw Gfx3DException("Could not load menubar.bmp - no valid texture format is supported","Gfx3D.cpp",__LINE__);
		return false;
	}

	//attempt to load the two cube textures.
	if ( SUCCEEDED( pD3D9->CheckDeviceFormat( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, fmtDisplay, 0, D3DRTYPE_TEXTURE, D3DFMT_X8R8G8B8) ) ) {
		//load both textures [256x256] cube1.bmp, cube2.bmp
		if( FAILED( D3DXCreateTextureFromFileEx( pDev, "cube1.bmp",
												 256, 256, 1, 0,
												 D3DFMT_X8R8G8B8, D3DPOOL_MANAGED,
												 D3DX_FILTER_LINEAR, D3DX_FILTER_LINEAR,
												 0, NULL, NULL, &pTexCube1 ) ) )
			 throw Gfx3DException("Could not load cube1.bmp as a 32bit texture","Gfx3D.cpp",__LINE__);

		if( FAILED( D3DXCreateTextureFromFileEx( pDev, "cube2.bmp",
												 256, 256, 1, 0,
												 D3DFMT_X8R8G8B8, D3DPOOL_MANAGED,
												 D3DX_FILTER_LINEAR, D3DX_FILTER_LINEAR,
												 0, NULL, NULL, &pTexCube2 ) ) )
			 throw Gfx3DException("Could not load cube2.bmp as a 32bit texture","Gfx3D.cpp",__LINE__);

	} else if ( SUCCEEDED( pD3D9->CheckDeviceFormat( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, fmtDisplay, 0, D3DRTYPE_TEXTURE, D3DFMT_R5G6B5 ) ) ) {
		//load both textures
		if( FAILED( D3DXCreateTextureFromFileEx( pDev, "cube1.bmp",
												 256, 256, 1, 0,
												 D3DFMT_R5G6B5, D3DPOOL_MANAGED,
												 D3DX_FILTER_LINEAR, D3DX_FILTER_LINEAR,
												 0, NULL, NULL, &pTexCube1 ) ) )
			 throw Gfx3DException("Could not load cube1.bmp as a 16bit texture","Gfx3D.cpp",__LINE__);

		if( FAILED( D3DXCreateTextureFromFileEx( pDev, "cube2.bmp",
												 256, 256, 1, 0,
												 D3DFMT_R5G6B5, D3DPOOL_MANAGED,
												 D3DX_FILTER_LINEAR, D3DX_FILTER_LINEAR,
												 0, NULL, NULL, &pTexCube2 ) ) )
			 throw Gfx3DException("Could not load cube2.bmp as a 16bit texture","Gfx3D.cpp",__LINE__);

	} else {
		//error!
		throw Gfx3DException("Could not load either cube texture - no valid texture format is supported","Gfx3D.cpp",__LINE__);
		return false;
	}
	return true;
}

bool Gfx3D::LoadGeometry() {

	const float fMBWidth = 158.0f;
	const float fMBHeight = 93.0f;

	if( FAILED(pDev->CreateVertexBuffer( 4*sizeof(vertex2D),0,FVF_VERTEX2D,D3DPOOL_MANAGED, &pVBMenu, NULL)) )
		throw Gfx3DException("Could not create 'pVBMenu' vertex buffer","Gfx3D.cpp",__LINE__);

	vertex2D* pVerts;
	pVBMenu->Lock(0,0,(void**)&pVerts,0);

		pVerts[0].p = D3DXVECTOR4(	0,	86,	0,	1	);
		pVerts[0].t = D3DXVECTOR2(	0,	2 / 127);

		pVerts[1].p = D3DXVECTOR4(	fMBWidth,	86,	0,	1	);
		pVerts[1].t = D3DXVECTOR2(	fMBWidth / 255,	2 / 127);

		pVerts[2].p = D3DXVECTOR4(	0,	86 + fMBHeight,	0,	1	);
		pVerts[2].t = D3DXVECTOR2(	0,	fMBHeight / 127);

		pVerts[3].p = D3DXVECTOR4(	fMBWidth,	86 + fMBHeight,	0,	1	);
		pVerts[3].t = D3DXVECTOR2(	fMBWidth / 255,	fMBHeight / 127);

	pVBMenu->Unlock();

	if ( FAILED(pDev->CreateVertexBuffer(36*sizeof(vertex3D),0,FVF_VERTEX3D,D3DPOOL_MANAGED,&pVBCube,NULL)))
		throw Gfx3DException("Could not create 'pVBCube' vertex buffer","Gfx3D.cpp",__LINE__);

	vertex3D* pCube;
	pVBCube->Lock(0,0,(void**)&pCube,0);
		//generate the top plane					
		pCube[ 0].p = D3DXVECTOR3(-0.5,0.5,0.5);
		pCube[ 0].t = D3DXVECTOR2(0,0);
		pCube[ 1].p = D3DXVECTOR3(0.5,0.5,0.5);
		pCube[ 1].t = D3DXVECTOR2(1,0);
		pCube[ 2].p = D3DXVECTOR3(-0.5,0.5,-0.5);
		pCube[ 2].t = D3DXVECTOR2(0,1);

		pCube[ 3].p = D3DXVECTOR3(0.5,0.5,-0.5);
		pCube[ 3].t = D3DXVECTOR2(1,1);
		pCube[ 4].p = D3DXVECTOR3(-0.5,0.5,-0.5);
		pCube[ 4].t = D3DXVECTOR2(0,1);
		pCube[ 5].p = D3DXVECTOR3(0.5,0.5,0.5);
		pCube[ 5].t = D3DXVECTOR2(1,0);

		//generate the bottom plane					
		pCube[ 6].p = D3DXVECTOR3(-0.5,-0.5,-0.5);
		pCube[ 6].t = D3DXVECTOR2(1,0);
		pCube[ 7].p = D3DXVECTOR3(0.5,-0.5,0.5);
		pCube[ 7].t = D3DXVECTOR2(0,1);
		pCube[ 8].p = D3DXVECTOR3(-0.5,-0.5,0.5);
		pCube[ 8].t = D3DXVECTOR2(0,0);

		pCube[ 9].p = D3DXVECTOR3(0.5,-0.5,0.5);
		pCube[ 9].t = D3DXVECTOR2(0,1);
		pCube[10].p = D3DXVECTOR3(-0.5,-0.5,-0.5);
		pCube[10].t = D3DXVECTOR2(1,0);
		pCube[11].p = D3DXVECTOR3(0.5,-0.5,-0.5);
		pCube[11].t = D3DXVECTOR2(1,1);

		//generate the left plane					
		pCube[12].p = D3DXVECTOR3(-0.5,0.5,-0.5);
		pCube[12].t = D3DXVECTOR2(0,0);
		pCube[13].p = D3DXVECTOR3(-0.5,-0.5,-0.5);
		pCube[13].t = D3DXVECTOR2(1,0);
		pCube[14].p = D3DXVECTOR3(-0.5,0.5,0.5);
		pCube[14].t = D3DXVECTOR2(0,1);

		pCube[15].p = D3DXVECTOR3(-0.5,0.5,0.5);
		pCube[15].t = D3DXVECTOR2(0,1);
		pCube[16].p = D3DXVECTOR3(-0.5,-0.5,-0.5);
		pCube[16].t = D3DXVECTOR2(1,0);
		pCube[17].p = D3DXVECTOR3(-0.5,-0.5,0.5);
		pCube[17].t = D3DXVECTOR2(1,1);

		//generate the right plane					
		pCube[18].p = D3DXVECTOR3(0.5,0.5,-0.5);
		pCube[18].t = D3DXVECTOR2(0,0);
		pCube[19].p = D3DXVECTOR3(0.5,0.5,0.5);
		pCube[19].t = D3DXVECTOR2(1,0);
		pCube[20].p = D3DXVECTOR3(0.5,-0.5,-0.5);
		pCube[20].t = D3DXVECTOR2(0,1);

		pCube[21].p = D3DXVECTOR3(0.5,-0.5,0.5);
		pCube[21].t = D3DXVECTOR2(1,1);
		pCube[22].p = D3DXVECTOR3(0.5,-0.5,-0.5);
		pCube[22].t = D3DXVECTOR2(0,1);
		pCube[23].p = D3DXVECTOR3(0.5,0.5,0.5);
		pCube[23].t = D3DXVECTOR2(1,0);

		//generate the back plane					
		pCube[24].p = D3DXVECTOR3(-0.5,0.5,-0.5);
		pCube[24].t = D3DXVECTOR2(0,0);
		pCube[25].p = D3DXVECTOR3(0.5,0.5,-0.5);
		pCube[25].t = D3DXVECTOR2(1,0);
		pCube[26].p = D3DXVECTOR3(-0.5,-0.5,-0.5);
		pCube[26].t = D3DXVECTOR2(0,1);

		pCube[27].p = D3DXVECTOR3(0.5,-0.5,-0.5);
		pCube[27].t = D3DXVECTOR2(1,1);
		pCube[28].p = D3DXVECTOR3(-0.5,-0.5,-0.5);
		pCube[28].t = D3DXVECTOR2(0,1);
		pCube[29].p = D3DXVECTOR3(0.5,0.5,-0.5);
		pCube[29].t = D3DXVECTOR2(1,0);

		//generate the front plane
		pCube[30].p = D3DXVECTOR3(0.5,0.5,0.5);
		pCube[30].t = D3DXVECTOR2(1,0);
		pCube[31].p = D3DXVECTOR3(-0.5,0.5,0.5);
		pCube[31].t = D3DXVECTOR2(0,0);
		pCube[32].p = D3DXVECTOR3(-0.5,-0.5,0.5);
		pCube[32].t = D3DXVECTOR2(0,1);

		pCube[33].p = D3DXVECTOR3(-0.5,-0.5,0.5);
		pCube[33].t = D3DXVECTOR2(0,1);
		pCube[34].p = D3DXVECTOR3(0.5,-0.5,0.5);
		pCube[34].t = D3DXVECTOR2(1,1);
		pCube[35].p = D3DXVECTOR3(0.5,0.5,0.5);
		pCube[35].t = D3DXVECTOR2(1,0);

	pVBCube->Unlock();

	return true;
}