using System;
using System.Drawing;
using System.Windows.Forms;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

using SystemFont = System.Drawing.Font;
using DXFont = Microsoft.DirectX.Direct3D.Font;

namespace DX9Test
{
	/// <summary>
	/// Summary description for GraphicsEngine.
	/// </summary>
	public sealed class GraphicsEngine : IDisposable
	{
		#region Member Variables
		// core objects
		private Device device;

		// transformation matrices
		private Matrix cube1Matrix;
		private Matrix cube2Matrix;
		//private Matrix viewMatrix;
		//private Matrix projMatrix;

		// textures
		private Texture cube1Texture;
		private Texture cube2Texture;
		private Texture menuTexture;

		// geometry
		private VertexBuffer cubeVerts;
		private VertexBuffer menuVerts;

		// fonts
		private DXFont outputFont;

		// per-frame variables
		private int lastFrameUpdate;
		private float cube1Angle;
		private float cube2Angle;

		// constants
		private const float cube1Speed = 50.0f;
		private const float cube2Speed = 75.0f;
		private const float cube1Size = 2.0f;
		private const float cube2Size = 4.0f;
		private const int FPSProfileSpeed = 200; //ms
		private const float menuBarWidth = 158;
		private const int fontSize = 11;
		private const float menuBarHeight = 93;

		// control variables
		private bool renderWireFrame = false;
		private bool renderTextures = true;
		private bool backFaceCulling = true;
		private bool drawFPS = true;
		private bool showFrameRate = true;

		// reference variables
		private Control target;
		private bool windowed;

		// misc variables
		private bool initOkay = false;
		private int lastFPSCheck;
		private int currCount;
		private int frameRate;
		private string devInfo;
		private string dispInfo;
		private bool disposed = false;
		#endregion

		#region Static Members
		public static bool IsDisplayModeOkay(int width, int height, int depth)
		{
			try
			{
				Format fmt = 0;

				switch (depth)
				{
					case 16:
						fmt = Format.R5G6B5;
						break;
					case 32:
						fmt = Format.X8R8G8B8;
						break;
				}

				foreach (AdapterInformation info in Manager.Adapters)
				{
					foreach (DisplayMode mode in info.SupportedDisplayModes[fmt])
					{
						if (mode.Width == width && mode.Height == height)
							return true;
					}
				}
			}
			catch (Exception)
			{
				return false;
			}

			return false;
		}
		#endregion

		#region Constructors
		/// <summary>
		///		Creates a windowed application based on the current display properties
		///		and that of the <paramref name="target"/> specified.
		/// </summary>
		/// <param name="target"></param>
		public GraphicsEngine(Control target)
		{
			try
			{
				PresentParameters d3dPP = new PresentParameters();

				d3dPP.Windowed = this.windowed = true;
				d3dPP.SwapEffect = SwapEffect.Discard;
				d3dPP.BackBufferCount = 1;
				d3dPP.BackBufferFormat = Manager.Adapters[0].CurrentDisplayMode.Format;
				d3dPP.BackBufferWidth = target.ClientSize.Width;
				d3dPP.BackBufferHeight = target.ClientSize.Height;

				this.dispInfo = String.Format("[Windowed] {0} x {1} {2}", d3dPP.BackBufferWidth, 
					d3dPP.BackBufferHeight, d3dPP.BackBufferFormat);

				this.target = target;

				InitializeDevice(target, d3dPP);
			}
			catch (Exception e)
			{
				this.initOkay = false;
				throw new ApplicationException("Could not initialize graphics engine.", e);
			}
		}

		/// <summary>
		///		Creates a fullscreen application based on the display properties
		///		specified. Throws an exception if the parameters are not valid.
		/// </summary>
		/// <param name="target"></param>
		/// <param name="width"></param>
		/// <param name="height"></param>
		/// <param name="depth"></param>
		public GraphicsEngine(Control target, int width, int height, int depth)
		{
			if (!IsDisplayModeOkay(width, height, depth))
			{
				throw new ApplicationException("Could not verify that the selected " +
					"resolution is compatible with the hardware.");
			}

			this.windowed = false;

			try
			{
				PresentParameters d3dPP = new PresentParameters();
				d3dPP.BackBufferWidth = width;
				d3dPP.BackBufferHeight = height;
				d3dPP.BackBufferCount = 1;
				d3dPP.SwapEffect = SwapEffect.Copy;
				d3dPP.PresentationInterval = PresentInterval.Immediate;

				this.dispInfo = String.Format("[Fullscreen] {0} x {1}", width, height);

				switch (depth)
				{
					case 16:
						d3dPP.BackBufferFormat = Format.R5G6B5;
						this.dispInfo += " 16bit R5G6B5";
						break;
					case 32:
						d3dPP.BackBufferFormat = Format.X8R8G8B8;
						this.dispInfo += " 32bit X8R8G8B8";
						break;
					default:
						throw new ApplicationException("Color depth is not valid - only 16 or 32 is acceptable.");
				}

				this.target = target;

				InitializeDevice(target, d3dPP);
			}
			catch (Exception e)
			{
				this.initOkay = false;
				throw new ApplicationException("Could not initialize graphics engine.", e);
			}
		}
		#endregion

		#region IDisposable Members

		private void Dispose(bool disposing)
		{
			if (!disposed)
			{
				if (disposing)
				{
					// release managed resources
					outputFont.Dispose();
					cubeVerts.Dispose();
					menuVerts.Dispose();
					menuTexture.Dispose();
					cube1Texture.Dispose();
					cube2Texture.Dispose();
					device.Dispose();
				}

				// release unmanaged resources

				disposed = true;
			}
		}

		public void Dispose()
		{
			Dispose(true);
			GC.SuppressFinalize(this);
		}

		~GraphicsEngine()
		{
			Dispose(false);
		}

		#endregion

		#region Properties

		/// <summary>
		///		Allows the host to specify if it wants to render
		///		in wireframe or not.
		/// </summary>
		public bool WireFrame
		{
			get { return this.renderWireFrame; }
			set
			{
				CheckState();
				this.renderWireFrame = value;

				if (value)
					device.RenderState.FillMode = FillMode.WireFrame;
				else
					device.RenderState.FillMode = FillMode.Solid;
			}
		}

		/// <summary>
		///		Allows the host to tell the engine to render
		///		with/without textures applied to the surfaces.
		/// </summary>
		public bool UseTextures
		{
			get { return this.renderTextures; }
			set
			{
				CheckState();
				this.renderTextures = value;
			}
		}

		/// <summary>
		///		Turns back face culling on or off.
		/// </summary>
		public bool BackFaceCulling
		{
			get { return this.backFaceCulling; }
			set
			{
				CheckState();
				this.backFaceCulling = value;

				if (value)
					device.RenderState.CullMode = Cull.CounterClockwise;
				else
					device.RenderState.CullMode = Cull.None;
			}
		}

		/// <summary>
		///		Allows the host to specify whether or not the frame rate
		///		should be drawn on the screen.
		/// </summary>
		public bool DrawFrameRate
		{
			get { return this.drawFPS; }
			set
			{
				CheckState();
				this.drawFPS = value;
			}
		}

		#endregion

		#region Public Methods
		/// <summary>
		///		This updates all the math, physics, and any other
		///		per-frame calculations necessary.
		/// </summary>
		public void OneFrameUpdate()
		{
			CheckState();

			// calculate the current rotation angles for the two cubes
			int currentTicks = Environment.TickCount;
			cube1Angle += ((currentTicks - lastFrameUpdate) / 1000f) * cube1Speed;
			cube2Angle += ((currentTicks - lastFrameUpdate) / 1000f) * cube2Speed;
			lastFrameUpdate = currentTicks;

			// calculate cube1's matrix
			cube1Matrix = Matrix.Identity;
			cube1Matrix *= Matrix.Scaling(3f, 3f, 3f);
			cube1Matrix *= Matrix.RotationX((float)(cube1Angle * (Math.PI / 180)));
			cube1Matrix *= Matrix.RotationY((float)(cube1Angle * (Math.PI / 180)));

			// calculate cube2's matrix
			cube2Matrix = Matrix.Identity;
			cube2Matrix *= Matrix.Scaling(4f, 5f, 0.75f);
			cube2Matrix *= Matrix.Translation(-8f, -4f, 0);
			cube2Matrix *= Matrix.RotationZ((float)(cube2Angle * (Math.PI / 180)));

			// calculate the current framerate
			if (currentTicks - lastFPSCheck >= FPSProfileSpeed)
			{
				lastFPSCheck = currentTicks;
				frameRate = currCount * (1000 / FPSProfileSpeed);
				currCount = 0;
			}
			currCount++;
		}

		/// <summary>
		///		This handles rendering of all graphics. It
		///		shouldn't have to deal with updating any variables
		///		or objects.
		/// </summary>
		public void OneFrameRender()
		{
			CheckState();

			// 1. Clear the render target
			device.Clear(ClearFlags.Target | ClearFlags.ZBuffer,
				Color.FromArgb(255, 0, 0, 64), 1.0f, 0);

			// 2. Begin rendering
			device.BeginScene();

			// 3. Render 3D objects
			device.SetStreamSource(0, cubeVerts, 0);
			device.VertexFormat = CustomVertex.PositionTextured.Format;

			// render the first cube
			device.SetTexture(0, (renderTextures ? cube1Texture : null));
			device.Transform.World = cube1Matrix;
			device.DrawPrimitives(PrimitiveType.TriangleList, 0, 12);

			// render the second cube
			device.SetTexture(0, (renderTextures ? cube2Texture : null));
			device.Transform.World = cube2Matrix;
			device.DrawPrimitives(PrimitiveType.TriangleList, 0, 12);

			// 4. Render 2D objects and text
			
			// render the menu background
			bool prevWireFrame = this.WireFrame;
			this.WireFrame = false;
			device.RenderState.AlphaBlendEnable = true;

			device.SetStreamSource(0, menuVerts, 0);
			device.VertexFormat = CustomVertex.TransformedTextured.Format;
			device.SetTexture(0, menuTexture);
			device.DrawPrimitives(PrimitiveType.TriangleStrip, 0, 2);

			device.RenderState.AlphaBlendEnable = false;
			this.WireFrame = prevWireFrame;

			DrawTextFormat textFormat = DrawTextFormat.Left | DrawTextFormat.Top;
			Color textColor = Color.FromArgb(255, 200, 128, 64);

			// draw the status text
			if (drawFPS)
			{
				outputFont.DrawText(devInfo, new Rectangle(5, 5, 0, 0),	textFormat,	textColor);

				outputFont.DrawText(dispInfo, new Rectangle(5, 5 + (fontSize + 6), 0, 0),
					textFormat,	textColor);

				outputFont.DrawText("Frame Rate: " + frameRate.ToString() + "fps",
					new Rectangle(5, 5 + (2 * (fontSize + 6)), 0, 0), textFormat, textColor);
			}

			// draw the menu bar
			outputFont.DrawText((renderWireFrame ? "F1 - Solid" : "F1 = Wireframe"),
				new Rectangle(5, 5 + (5 * (fontSize + 6)), 0, 0), textFormat, textColor);

			outputFont.DrawText((renderTextures ? "F2 - Shaded" : "F2 - Textured"),
				new Rectangle(5, 5 + (6 * (fontSize + 6)), 0, 0), textFormat, textColor);

			outputFont.DrawText((backFaceCulling ? "F3 - No Culling" : "F3 - Cull Back Faces"),
				new Rectangle(5, 5 + (7 * (fontSize + 6)), 0, 0), textFormat, textColor);

			outputFont.DrawText((drawFPS ? "F4 - Hide Status" : "F4 - Show Status"),
				new Rectangle(5, 5 + (8 * (fontSize + 6)), 0, 0), textFormat, textColor);

			outputFont.DrawText("ESC - Quit Demo", 
				new Rectangle(5, 5 + (9 * (fontSize + 6)), 0, 0), textFormat, textColor);

			// 5. End rendering
			device.EndScene();

			// 6. Present frame on screen
			device.Present();
		}
		#endregion

		#region Private Methods
		/// <summary>
		///		Once the constructor has filled out the PresentParameters,
		///		we can use this method to finish off all device initialization.
		/// </summary>
		/// <param name="target"></param>
		/// <param name="parameters"></param>
		private void InitializeDevice(Control target, PresentParameters parameters)
		{
			// 1. Check for, and specify, depth buffer
			if (Manager.CheckDepthStencilMatch(0, DeviceType.Hardware, 
				parameters.BackBufferFormat, parameters.BackBufferFormat, 
				DepthFormat.D16))
			{
				// support exists
				parameters.AutoDepthStencilFormat = DepthFormat.D16;
				parameters.EnableAutoDepthStencil = true;
			}
			else
			{
				throw new ApplicationException("Direct3D adapter does not support 16bit depth buffers. " +
					"This sample requires a depth buffer with at least 16 bits precision.");
			}

			// 2. Examine the device capabilities
			Caps d3dCaps = Manager.GetDeviceCaps(0, DeviceType.Hardware);

			CreateFlags flags = CreateFlags.MultiThreaded;

			// allow this engine to take advantage of hardware transform and lighting if available
			if (d3dCaps.DeviceCaps.SupportsHardwareTransformAndLight)
				flags |= CreateFlags.HardwareVertexProcessing;
			else
				flags |= CreateFlags.SoftwareVertexProcessing;

			// 3. Attempt to create the device interface
			this.device = new Device(0, DeviceType.Hardware, target, flags, parameters);

			// 4. Configure render states and other parameters
			// We need to tell D3D we want it to use the depth buffer
			device.RenderState.ZBufferEnable = true;

			// we don't want to use lighting in this sample
			device.RenderState.Lighting = false;

			// allow texture transparencies
			device.RenderState.SourceBlend = Blend.SourceAlpha;
			device.RenderState.DestinationBlend = Blend.InvSourceAlpha;

			// set up texture blending
			device.SamplerState[0].MinFilter = TextureFilter.Linear;
			device.SamplerState[0].MagFilter = TextureFilter.Linear;

			// 5. Load textures
			LoadTextures(parameters.BackBufferFormat);

			// 6. Create the necessary geometry
			LoadGeometry();

			// 7. Set up the matrices
			// view matrix: describes the properties of the camera
			device.Transform.View = Matrix.LookAtLH(new Vector3(0f, 0f, -15f),
													new Vector3(0f, 0f, 0f),
													new Vector3(0f, 1f, 0f));

			// projection matrix: describes the properties of the camera lens
			device.Transform.Projection = Matrix.PerspectiveFovLH((float)(Math.PI / 4), (4f / 3f), 1f, 100f);

			// world matrix: how all the geometry is altered in world space
			device.Transform.World = Matrix.Identity;

			// 8. Sort out font sized for on-screen rendering
			this.outputFont = new DXFont(device, new SystemFont("Arial", (float)fontSize, FontStyle.Bold));
			
			// 9. Configure misc variables

			// If we don't do this then the initial angle calculations go pear shaped.
			this.lastFrameUpdate = Environment.TickCount;

			// store the name of the graphics card as the driver reports it
			this.devInfo = Manager.Adapters[0].Information.Description;

			// if we got this far then we can assume it was a successful initialization
			this.initOkay = true;
		}

		/// <summary>
		///		This loads all the required textures from disk.
		/// </summary>
		/// <param name="adapterFormat"></param>
		private void LoadTextures(Format adapterFormat)
		{
			// attempt to load the menu texture
			if (Manager.CheckDeviceFormat(0, DeviceType.Hardware, adapterFormat, 0, 
				ResourceType.Textures, Format.A8R8G8B8))
			{
				menuTexture = TextureLoader.FromFile(device, Application.StartupPath + "\\menubar.bmp",
					256, 128, 1, 0, Format.A8R8G8B8, Pool.Managed,
					Filter.None, Filter.None, Color.Magenta.ToArgb());
			}
			else if (Manager.CheckDeviceFormat(0, DeviceType.Hardware, adapterFormat, 0,
				ResourceType.Textures, Format.A1R5G5B5))
			{
				menuTexture = TextureLoader.FromFile(device, Application.StartupPath + "\\menubar.bmp",
					256, 128, 1, 0, Format.A1R5G5B5, Pool.Managed,
					Filter.None, Filter.None, Color.Magenta.ToArgb());
			}
			else
			{
				throw new ApplicationException("Menu texture cannot be created: no support for 16 bit or 32 bit textures with at least 1 bit of alpha");
			}

			// attempt to load the cube textures
			if (Manager.CheckDeviceFormat(0, DeviceType.Hardware, adapterFormat, 0, 
				ResourceType.Textures, Format.X8R8G8B8))
			{
				cube1Texture = TextureLoader.FromFile(device, Application.StartupPath + "\\cube1.bmp",
					256, 256, 1, 0, Format.X8R8G8B8, Pool.Managed, Filter.Linear, Filter.Linear, 0);

				cube2Texture = TextureLoader.FromFile(device, Application.StartupPath + "\\cube2.bmp",
					256, 256, 1, 0, Format.X8R8G8B8, Pool.Managed, Filter.Linear, Filter.Linear, 0);
			}
			else if (Manager.CheckDeviceFormat(0, DeviceType.Hardware, adapterFormat, 0,
				ResourceType.Textures, Format.R5G6B5))
			{
				cube1Texture = TextureLoader.FromFile(device, Application.StartupPath + "\\cube1.bmp",
					256, 256, 1, 0, Format.R5G6B5, Pool.Managed, Filter.Linear, Filter.Linear, 0);

				cube2Texture = TextureLoader.FromFile(device, Application.StartupPath + "\\cube2.bmp",
					256, 256, 1, 0, Format.R5G6B5, Pool.Managed, Filter.Linear, Filter.Linear, 0);
			}
			else
			{
				throw new ApplicationException("Cube textures cannot be created: no support for 16 bit or 32 bit textures.");
			}
		}

		/// <summary>
		///		This will create and/or load any geometry that we're going to be using.
		/// </summary>
		private void LoadGeometry()
		{
			// load 2D geometry
			menuVerts = new VertexBuffer(typeof(CustomVertex.TransformedTextured), 4, device, 0,
				CustomVertex.TransformedTextured.Format, Pool.Default);

			CustomVertex.TransformedTextured[] v = (CustomVertex.TransformedTextured[])menuVerts.Lock(0, 0);

			// Use the constructors to fill the data
			// laid out such that first line = coordinate (x,y,z,rhw)
			// and second line = texcoord (u,v)
			v[0] = new CustomVertex.TransformedTextured(0f, 86f, 0f, 1f, 0f, 2f / 127f);
			v[1] = new CustomVertex.TransformedTextured(menuBarWidth, 86, 0, 1, menuBarWidth / 255f, 2f / 127f);
			v[2] = new CustomVertex.TransformedTextured(0f, 86f + menuBarHeight, 0f, 1f, 0f, menuBarHeight / 127f);
			v[3] = new CustomVertex.TransformedTextured(menuBarWidth, 86f + menuBarHeight, 0f, 1f, menuBarWidth / 255f, menuBarHeight / 127f);

			menuVerts.Unlock();

			// load 3D geometry
			cubeVerts = new VertexBuffer(typeof(CustomVertex.PositionTextured), 36, device, 0,
				CustomVertex.PositionTextured.Format, Pool.Managed);

			CustomVertex.PositionTextured[] vCube = (CustomVertex.PositionTextured[])cubeVerts.Lock(0, 0);

			// 2a. Generate a top plane
			vCube[0] = new CustomVertex.PositionTextured(-0.5f, 0.5f, 0.5f, 0f, 0f);
			vCube[1] = new CustomVertex.PositionTextured(0.5f, 0.5f, 0.5f, 1f, 0f);
			vCube[2] = new CustomVertex.PositionTextured(-0.5f, 0.5f, -0.5f, 0f, 1f);

			vCube[3] = new CustomVertex.PositionTextured(0.5f, 0.5f, -0.5f, 1f, 1f);
			vCube[4] = new CustomVertex.PositionTextured(-0.5f, 0.5f, -0.5f, 0f, 1f);
			vCube[5] = new CustomVertex.PositionTextured(0.5f, 0.5f, 0.5f, 1f, 0f);

			// 2b. Generate bottom plane
			vCube[6] = new CustomVertex.PositionTextured(-0.5f, -0.5f, -0.5f, 1f, 0f);
			vCube[7] = new CustomVertex.PositionTextured(0.5f, -0.5f, 0.5f, 0f, 1f);
			vCube[8] = new CustomVertex.PositionTextured(-0.5f, -0.5f, 0.5f, 0f, 0f);

			vCube[9] = new CustomVertex.PositionTextured(0.5f, -0.5f, 0.5f, 0f, 1f);
			vCube[10] = new CustomVertex.PositionTextured(-0.5f, -0.5f, -0.5f, 1f, 0f);
			vCube[11] = new CustomVertex.PositionTextured(0.5f, -0.5f, -0.5f, 1f, 1f);

			// 2c. Generate left plane
			vCube[12] = new CustomVertex.PositionTextured(-0.5f, 0.5f, -0.5f, 0f, 0f);
			vCube[13] = new CustomVertex.PositionTextured(-0.5f, -0.5f, -0.5f, 1f, 0f);
			vCube[14] = new CustomVertex.PositionTextured(-0.5f, 0.5f, 0.5f, 0f, 1f);

			vCube[15] = new CustomVertex.PositionTextured(-0.5f, 0.5f, 0.5f, 0f, 1f);
			vCube[16] = new CustomVertex.PositionTextured(-0.5f, -0.5f, -0.5f, 1f, 0f);
			vCube[17] = new CustomVertex.PositionTextured(-0.5f, -0.5f, 0.5f, 1f, 1f);

			// 2d. Generate right plane
			vCube[18] = new CustomVertex.PositionTextured(0.5f, 0.5f, -0.5f, 0f, 0f);
			vCube[19] = new CustomVertex.PositionTextured(0.5f, 0.5f, 0.5f, 1f, 0f);
			vCube[20] = new CustomVertex.PositionTextured(0.5f, -0.5f, -0.5f, 0f, 1f);

			vCube[21] = new CustomVertex.PositionTextured(0.5f, -0.5f, 0.5f, 1f, 1f);
			vCube[22] = new CustomVertex.PositionTextured(0.5f, -0.5f, -0.5f, 0f, 1f);
			vCube[23] = new CustomVertex.PositionTextured(0.5f, 0.5f, 0.5f, 1f, 0f);

			// 2e. Generate back plane
			vCube[24] = new CustomVertex.PositionTextured(-0.5f, 0.5f, -0.5f, 0f, 0f);
			vCube[25] = new CustomVertex.PositionTextured(0.5f, 0.5f, -0.5f, 1f, 0f);
			vCube[26] = new CustomVertex.PositionTextured(-0.5f, -0.5f, -0.5f, 0f, 1f);

			vCube[27] = new CustomVertex.PositionTextured(0.5f, -0.5f, -0.5f, 1f, 1f);
			vCube[28] = new CustomVertex.PositionTextured(-0.5f, -0.5f, -0.5f, 0f, 1f);
			vCube[29] = new CustomVertex.PositionTextured(0.5f, 0.5f, -0.5f, 1f, 0f);

			// 2f. Generate front plane
			vCube[30] = new CustomVertex.PositionTextured(0.5f, 0.5f, 0.5f, 1f, 0f);
			vCube[31] = new CustomVertex.PositionTextured(-0.5f, 0.5f, 0.5f, 0f, 0f);
			vCube[32] = new CustomVertex.PositionTextured(-0.5f, -0.5f, 0.5f, 0f, 1f);

			vCube[33] = new CustomVertex.PositionTextured(-0.5f, -0.5f, 0.5f, 0f, 1f);
			vCube[34] = new CustomVertex.PositionTextured(0.5f, -0.5f, 0.5f, 1f, 1f);
			vCube[35] = new CustomVertex.PositionTextured(0.5f, 0.5f, 0.5f, 1f, 0f);

			cubeVerts.Unlock();
		}

		private void CheckState()
		{
			if (!initOkay)
				throw new ApplicationException("Cannot update frame: engine was not properly initialized.");

			if (disposed)
				throw new ObjectDisposedException("GraphicsEngine");
		}
		#endregion
	}
}
